import logging
from typing import Optional
import sys


def config_logs(
    level: int = logging.WARNING,
    stdout_level: Optional[int] = None,
    stderr_level: Optional[int] = logging.WARNING,
    handlers: list = [],
):
    if stdout_level is not None:
        stdout = logging.StreamHandler(sys.stdout)
        stdout.setLevel(stdout_level)
        handlers.append(stdout)
    if stderr_level is not None:
        stderr = logging.StreamHandler(sys.stderr)
        stderr.setLevel(stderr_level)
        handlers.append(stderr)
    logging.basicConfig(handlers=handlers, force=True)



import os
from pathlib import Path
from typing import Optional, Union

import tableauserverclient as TSC


class MultipleProjectsShareNameException(Exception):
    def __init__(self, name_shared, projects: Optional[list[TSC.ProjectItem]] = None):
        messages: list[str] = [
            f'Multiple projects share the name "{name_shared}",',
            "try providing a project name including parent projects separated by '/'",
            "or try referencing using the id of the project instead:",
        ]
        if projects is not None:
            project_descs = [
                f"  name: {x.name}; id: {x.id}; desc: {x.description}" for x in projects
            ]
            messages.extend(project_descs)

        message = "\n".join(messages)
        super().__init__(message)


class ProjectNotFoundException(Exception):
    def __init__(self, name_or_id):
        message = f'No project was found with the given name or id: "{name_or_id}",'
        super().__init__(message)


class ServerHelper:
    def __init__(
        self,
        tableau_server_url: Optional[str] = None,
        tableau_api_version: Optional[str] = None,
        verify: Union[bool, str] = True,
        token_name: Optional[str] = None,
        personal_access_token: Optional[str] = None,
    ):
        self.server: TSC.Server = ServerHelper._open_connection(
            tableau_server_url=tableau_server_url,
            tableau_api_version=tableau_api_version,
            verify=verify,
            token_name=token_name,
            personal_access_token=personal_access_token,
        )

    @staticmethod
    def _open_connection(
        tableau_server_url: Optional[str] = None,
        tableau_api_version: Optional[str] = None,
        verify: Union[bool, str] = True,
        token_name: Optional[str] = None,
        personal_access_token: Optional[str] = None,
    ):
        if tableau_server_url is None:
            tableau_server_url = os.getenv("TABLEAU_SERVER_URL")
        if token_name is None:
            token_name = os.getenv("TABLEAU_TOKEN_NAME")
        if personal_access_token is None:
            personal_access_token = os.getenv("TABLEAU_PERSONAL_ACCESS_TOKEN")

        tableau_ssl_cert = os.getenv("TABLEAU_SSL_CERT_PATH", None)
        if tableau_ssl_cert is not None and verify is True:
            verify = tableau_ssl_cert

        server = TSC.Server(tableau_server_url)

        if isinstance(verify, str):
            server.add_http_options({"verify": verify})
        if tableau_api_version is None:
            tableau_api_version = os.getenv("TABLEAU_API_VERSION", None)
        if tableau_api_version is None:
            server.use_server_version()
        else:
            server.version = tableau_api_version

        auth = TSC.PersonalAccessTokenAuth(
            token_name=token_name,
            personal_access_token=personal_access_token,
            site_id=os.getenv("TABLEAU_SITE", "")
        )
        server.auth.sign_in_with_personal_access_token(auth)
        return server

    def get_project_id(
        self,
        project_name: str,
        parent_id: Optional[str] = None,
    ) -> str:
        server = self.server
        projects: list[TSC.ProjectItem]
        project_name_remainder = None
        if project_name.find("/") > 0:
            project_name_parts = project_name.split("/", 1)
            project_name = project_name_parts[0]
            project_name_remainder = project_name_parts[1]
        req_option = TSC.RequestOptions()
        proj_filter = TSC.Filter(
            TSC.RequestOptions.Field.Name,
            TSC.RequestOptions.Operator.Equals,
            project_name,
        )
        req_option.filter.add(proj_filter)
        projects, _ = server.projects.get(req_option)
        if parent_id is not None:
            projects = [x for x in projects if x.parent_id == parent_id]
        # it isn't clear if tableau can return multiple projects with the same name
        # we should avoid unintentional writing to the wrong project
        if len(projects) > 1:
            raise MultipleProjectsShareNameException(project_name, projects)
        elif len(projects) < 1:
            # try finding a matching id instead of name as a backup
            projects = [x for x in projects if x.id == project_name]

        if len(projects) < 1:
            raise ProjectNotFoundException(project_name)

        project_id = projects[0].id
        if project_name_remainder is None:
            if project_id is None:
                raise ValueError("Error getting project ID")
            return project_id

        return self.get_project_id(project_name_remainder, project_id)

    def get_flow(self, project_name, flow_name) -> Optional[TSC.FlowItem]:
        project_id: str = self.get_project_id(project_name)
        flows: list[TSC.FlowItem]
        req_option = TSC.RequestOptions()
        flow_filter = TSC.Filter(
            TSC.RequestOptions.Field.Name,
            TSC.RequestOptions.Operator.Equals,
            flow_name,
        )
        req_option.filter.add(flow_filter)
        flows, _ = self.server.flows.get(req_option)
        for flow in flows:
            if flow.project_id == project_id:
                return flow
        return None

    def create_or_replace_hyper_file(
        self,
        source: Path,
        dest_project: str,
        name: Optional[str] = None,
    ):
        """

        :param source: the hyperfile to be uploaded
        :param dest_project: the project name, path, or id
        :param name: optionally set a different name from the filename
        :return: None
        """  # noqa
        project_id = self.get_project_id(
            project_name=dest_project,
        )
        server = self.server
        publish_mode = TSC.Server.PublishMode.Overwrite
        datasource = TSC.DatasourceItem(project_id, name=name)
        server.datasources.publish(datasource, source, publish_mode)


def get_project_id(
    project_name: str,
    parent_id: Optional[str] = None,
    tableau_server_url: Optional[str] = None,
    verify: Union[bool, str] = True,
    token_name: Optional[str] = None,
    personal_access_token: Optional[str] = None,
) -> Optional[str]:
    helper = ServerHelper(
        tableau_server_url=tableau_server_url,
        verify=verify,
        token_name=token_name,
        personal_access_token=personal_access_token,
    )
    return helper.get_project_id(project_name=project_name, parent_id=parent_id)


def create_or_replace_hyper_file(
    hyperfile_path: Path,
    project: str,
    name: Optional[str] = None,
    tableau_server_url: Optional[str] = None,
    verify: Union[bool, str] = True,
    token_name: Optional[str] = None,
    personal_access_token: Optional[str] = None,
):
    """

    :param hyperfile_path: the hyperfile to be uploaded
    :param project: the project name or id
    :param name: optionally set a different name from the filename
    :param tableau_server_url: the full url to your tableau_server
        (https://...), when None the ENVVAR "TABLEAU_SERVER_URL"
        will be loaded.
    :param verify: optionally disable ssl or point to a trusted
        cert path
    :param token_name: the name of your PAC, when None the
        ENVVAR "TABLEAU_TOKEN_NAME" will be loaded
    :param personal_access_token: the value of your PAC, when None
        the ENVVAR "TABLEAU_PERSONAL_ACCESS_TOKEN" will be loaded
    :return: None
    """  # noqa
    helper = ServerHelper(
        tableau_server_url=tableau_server_url,
        verify=verify,
        token_name=token_name,
        personal_access_token=personal_access_token,
    )
    helper.create_or_replace_hyper_file(
        source=hyperfile_path,
        dest_project=project,
        name=name,
    )



import os

import pytest
from dotenv import load_dotenv
from tableauhyperapi import SqlType, TableDefinition, TableName


@pytest.fixture(autouse=True)
def load_env():
    load_dotenv()


def good_csv_path():
    return os.path.abspath("tests/resources/good_default_format.csv")


def good_csv2_path():
    return os.path.abspath("tests/resources/good_default_format2.csv")


def good_left_path():
    return os.path.abspath("tests/resources/good_left.csv")


def good_right_path():
    return os.path.abspath("tests/resources/good_left.csv")


def good_tab_path():
    return os.path.abspath("tests/resources/good_default_format.tab")


def doesnt_exist_path():
    return os.path.abspath("tests/resources/doesnt_exist.csv")


def good_csv_schema():
    return TableDefinition(
        table_name=TableName("test_table_name"),
        columns=[
            TableDefinition.Column("integers", SqlType.int()),
            TableDefinition.Column("text1", SqlType.text()),
            TableDefinition.Column("text2", SqlType.text()),
        ],
    )


def bad_csv_schema():
    return TableDefinition(
        table_name=TableName("test_table_name"),
        columns=[
            TableDefinition.Column("integers", SqlType.int()),
            TableDefinition.Column("text1", SqlType.int()),
            TableDefinition.Column("text2", SqlType.text()),
        ],
    )


def good_table_def_path():
    return os.path.abspath("tests/resources/good_default_format_table_def.json")


def good_join_table_def_path():
    return os.path.abspath("tests/resources/good_join_table_def.json")


def good_hyper_path():
    return os.path.abspath("tests/resources/good_default_format.hyper")


def joined_hyper_path():
    return os.path.abspath("tests/resources/joined.hyper")



from pathlib import Path

import pytest
from tableauserverclient.server import FlowItem

import tests.conftest as ct
from tableau_helpers.server import ServerHelper


@pytest.mark.integration
def test_get_project_id():
    project_path = "ApiTest"
    helper = ServerHelper()
    helper.get_project_id(project_path)


@pytest.mark.integration
def test_get_project_id_nested():
    project_path = "ApiTest/Nested"
    helper = ServerHelper()
    helper.get_project_id(project_path)


@pytest.mark.integration
def test_create_or_replace_hyperfile(good_hyper_path=Path(ct.good_hyper_path())):
    helper = ServerHelper()
    helper.create_or_replace_hyper_file(good_hyper_path, "ApiTest", "test-data")


@pytest.mark.integration
def test_create_or_replace_joined_hyperfile(hyper_path=Path(ct.joined_hyper_path())):
    helper = ServerHelper()
    helper.create_or_replace_hyper_file(hyper_path, "ApiTest", "test-data")


@pytest.mark.integration
def test_get_flow_by_name():
    helper = ServerHelper()
    flow: FlowItem = helper.get_flow("ApiTest", "New Flow")
    assert flow is not None



from pathlib import Path

import pytest

from tableau_helpers.cli.csv_to_hyper import main
from tests import conftest


@pytest.mark.parametrize(
    "csv,table_def", [(conftest.good_csv_path(), conftest.good_table_def_path())]
)
def test_csv_to_hyper(tmp_path, csv, table_def):
    args = [
        "--csv",
        csv,
        "--table-def",
        table_def,
        "--dest",
        Path(tmp_path, "tmp.hyper").absolute().as_posix(),
    ]
    main(args)


@pytest.mark.parametrize(
    "tsv,table_def", [(conftest.good_tab_path(), conftest.good_table_def_path())]
)
def test_tsv_to_hyper(tmp_path, tsv, table_def):
    args = [
        "--csv",
        tsv,
        "--table-def",
        table_def,
        "--dest",
        Path(tmp_path, "tmp.hyper").absolute().as_posix(),
        "--delimiter",
        "\t",
    ]
    main(args)



import pytest

from tableau_helpers.cli.upload_hyper import main
from tests import conftest


@pytest.mark.parametrize("hyperfile,project", [(conftest.good_hyper_path(), "ApiTest")])
def test_upload_hyper(hyperfile, project):
    args = [
        "--hyperfile",
        hyperfile,
        "--dest",
        project,
    ]
    main(args)


@pytest.mark.parametrize(
    "hyperfile,project,new_name", [(conftest.good_hyper_path(), "ApiTest", "new-name")]
)
def test_upload_hyper_rename(hyperfile, project, new_name):
    args = [
        "--hyperfile",
        hyperfile,
        "--dest",
        project,
        "--new-name",
        new_name,
    ]
    main(args)



# Sample Test passing with nose and pytest
from pathlib import Path

import pytest
from tableauhyperapi import HyperException

import tests.conftest as ct
from tableau_helpers import hyper


def test_create_hyper_from_default_csv_path_obj(
    tmp_path, csv_path=ct.good_csv_path(), csv_schema=ct.good_csv_schema()
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")
    linecount = hyper.copy_csv_to_hyper(tmp_hyper_file, Path(csv_path), csv_schema)
    assert linecount == 2


def test_create_hyper_from_default_csv_path_str(
    tmp_path, csv_path=ct.good_csv_path(), csv_schema=ct.good_csv_schema()
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")
    linecount = hyper.copy_csv_to_hyper(tmp_hyper_file, Path(csv_path), csv_schema)
    assert linecount == 2


def test_create_hyper_from_default_tab_path_str(
    tmp_path, csv_path=ct.good_tab_path(), csv_schema=ct.good_csv_schema()
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")
    linecount = hyper.copy_csv_to_hyper(
        tmp_hyper_file, Path(csv_path), csv_schema, delimiter="\t"
    )
    assert linecount == 2


def test_fail_create_hyper_from_default_tab_path_str(
    tmp_path, csv_path=ct.good_tab_path(), csv_schema=ct.good_csv_schema()
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")
    with pytest.raises(HyperException):
        assert hyper.copy_csv_to_hyper(tmp_hyper_file, Path(csv_path), csv_schema)


def compare_columns(columns_a, columns_b):
    for x, y in zip(columns_a, columns_b):
        assert x.name == y.name
        assert x.nullability == y.nullability
        assert x.type == y.type
        assert x.collation == y.collation


def test_load_good_table_def(
    table_def_path=ct.good_table_def_path(), csv_schema=ct.good_csv_schema()
):
    table_def = hyper.load_table_defs(Path(table_def_path))
    assert table_def[0].table_name == csv_schema.table_name
    compare_columns(table_def[0].columns, csv_schema.columns)


def test_create_hyper_with_fact_table_str(
    tmp_path,
    csv_left=ct.good_left_path(),
    csv_right=ct.good_right_path(),
    csv_join_tabledef=ct.good_join_table_def_path(),
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")
    csvs = [Path(csv_left), Path(csv_right)]
    tabledef = hyper.load_table_defs(Path(csv_join_tabledef))
    linecount = hyper.copy_csv_to_hyper(tmp_hyper_file, csvs, tabledef)
    assert linecount == 4


def test_create_hyper_from_default_csv_doesnt_exist(
    tmp_path,
    doesnt_exist_path=ct.doesnt_exist_path(),
    good_csv_schema=ct.good_csv_schema(),
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")
    with pytest.raises(FileNotFoundError):
        assert hyper.copy_csv_to_hyper(
            tmp_hyper_file, Path(doesnt_exist_path), good_csv_schema
        )


def test_create_hyper_from_string_path(
    tmp_path,
    doesnt_exist_path=ct.doesnt_exist_path(),
    good_csv_schema=ct.good_csv_schema(),
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")
    with pytest.raises(TypeError):
        assert hyper.copy_csv_to_hyper(
            tmp_hyper_file, doesnt_exist_path, good_csv_schema
        )


def test_create_hyper_from_default_csv_with_header(
    tmp_path, good_csv_path=ct.good_csv_path(), good_csv_schema=ct.good_csv_schema()
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")
    linecount = hyper.copy_csv_to_hyper(
        tmp_hyper_file,
        Path(good_csv_path),
        good_csv_schema,
        header=True,
    )
    assert linecount == 1


def test_create_hyper_from_default_csv_bad_schema(
    tmp_path, good_csv_path=ct.good_csv_path(), bad_csv_schema=ct.bad_csv_schema()
):
    tmp_hyper_file = Path(tmp_path, "tmp.hyper")

    with pytest.raises(HyperException):
        assert hyper.copy_csv_to_hyper(
            tmp_hyper_file, Path(good_csv_path), bad_csv_schema
        )



import argparse
from pathlib import Path
import sys

from tableau_helpers import server


def _parse_args(args) -> argparse.Namespace:
    description = "Upload or overwrite a hyperfile on tableau-server"
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument(
        "--hyperfile",
        type=Path,
        required=True,
        help="The path to the hyperfile to be uploaded.",
    )
    parser.add_argument(
        "--dest",
        type=str,
        required=True,
        help="The destination project string from root folder",
    )
    parser.add_argument(
        "--new-name",
        type=str,
        required=False,
        default=None,
        help="A name for the datasource (if desired different from hyperfile)",
    )
    return parser.parse_args(args)


def main(args):
    args = _parse_args(args)
    server.create_or_replace_hyper_file(
        project=args.dest, hyperfile_path=args.hyperfile, name=args.new_name
    )


def entry_point():
    main(sys.argv[1:])


if __name__ == "__main__":
    entry_point()



import lzma
from pathlib import Path
import shutil
import tempfile
from typing import Union
from urllib.request import urlopen

tempdir = tempfile.TemporaryDirectory()
tmppath = Path(tempdir.name)


def download_file(url: str, dir: Path = tmppath) -> Path:
    filename = url.rsplit("/", 1)[-1]
    filepath = Path(dir, filename)
    with urlopen(url) as response:
        with filepath.open(mode="w+b") as f:
            shutil.copyfileobj(response, f)
    return filepath


def try_unzip(arg: Path):
    tmp = Path(tempdir.name, arg.with_suffix("").name)
    if arg.suffix == ".xz":
        with lzma.open(arg) as decompressed:
            with tmp.open(mode="w+b") as f:
                shutil.copyfileobj(decompressed, f)
        return try_unzip(tmp)
    return arg


def path_or_url(arg: Union[Path, str]) -> Path:
    result = arg
    if isinstance(arg, Path):
        result = arg
    if isinstance(arg, str):
        if arg.startswith("https://"):
            result = download_file(url=arg)
        else:
            result = Path(arg)
    else:
        raise TypeError

    return result



import argparse
import sys
from pathlib import Path
import tempfile

from tableau_helpers import hyper, utils

tempdir = tempfile.TemporaryDirectory()


def literal(value: str) -> str:
    return value.encode().decode("unicode_escape")


def _parse_args(args):
    description = (
        "csv_to_hyper creates a hyperfile with Tableau's hyperapi given a csv and"
        " table-definition"
    )
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument(
        "--csv",
        type=utils.path_or_url,
        required=True,
        nargs="+",
        help="Provide the CSV to be copied with the hyperapi.",
    )
    parser.add_argument(
        "--table-def",
        type=Path,
        required=True,
        help="Provide the path to the table-definition file for the csv.",
    )
    dest_help = (
        "Optionally provide the destination path for hyper, the default appends .hyper"
        " to the csv's path"
    )
    parser.add_argument("--dest", type=Path, required=False, help=dest_help)
    parser.add_argument(
        "--header",
        action="store_true",
        required=False,
        help="Specify if there is a header in your csv.",
    )
    parser.add_argument(
        "--delimiter",
        type=literal,
        default=",",
        required=False,
        help="Specify the delimiter in your csv.",
    )
    parser.add_argument(
        "--quote",
        type=str,
        default='"',
        required=False,
        help="Specify the quotation character in your csv.",
    )
    parser.add_argument(
        "--null",
        type=str,
        default="",
        required=False,
        help="Specify the value representing null entries in your csv.",
    )
    parser.add_argument(
        "--encoding",
        type=str,
        default="utf-8",
        required=False,
        help="Specify the encoding type of your csv.",
    )
    return parser.parse_args(args)


def main(args):
    args = _parse_args(args)
    args.csv = [utils.try_unzip(csv) for csv in args.csv]
    table_definition = hyper.load_table_defs(args.table_def)
    hyper.copy_csv_to_hyper(
        args.dest,
        args.csv,
        table_definition,
        delimiter=args.delimiter,
        header=args.header,
        quote=args.quote,
        null=args.null,
        encoding=args.encoding,
    )


def entry_point():
    main(sys.argv[1:])


if __name__ == "__main__":
    entry_point()



import json
import logging
import os
from pathlib import Path
from typing import Iterable, Optional, Union

from tableauhyperapi import (
    Connection,
    CreateMode,
    HyperProcess,
    Nullability,
    SqlType,
    TableDefinition,
    TableName,
    Telemetry,
    TypeTag,
    escape_string_literal,
)

logger = logging.getLogger(__name__)


def _gen_sql_for_with_csv(
    delimiter: str = ",",
    null: str = "",
    encoding: str = "utf-8",
    on_cast_failure: str = "error",
    header: bool = False,
    quote: str = '"',
    escape: Optional[str] = None,
    force_not_null: Optional[list[str]] = None,
    force_null: Optional[list[str]] = None,
):
    sql_with: str = "WITH ("
    sql_with += "FORMAT csv"
    sql_with += f", NULL {escape_string_literal(null)}"
    sql_with += f", DELIMITER {escape_string_literal(delimiter)}"
    sql_with += f", ENCODING {escape_string_literal(encoding)}"
    sql_with += f", ON_CAST_FAILURE {escape_string_literal(on_cast_failure)}"
    if header:
        sql_with += f", HEADER"
    sql_with += f", QUOTE {escape_string_literal(quote)}"
    if escape is not None:
        sql_with += f", ESCAPE {escape_string_literal(escape)}"
    if force_not_null is not None:
        sql_with += f", FORCE_NOT_NULL ({','.join(force_not_null)})"
    if force_null is not None:
        sql_with += f", FORCE_NULL ({','.join(force_null)})"
    sql_with += ")"
    return sql_with


def load_table_defs(table_conf_file: Path) -> list[TableDefinition]:
    with open(table_conf_file) as file:
        table_confs = json.load(file)
    return [parse_table_definition(table_conf) for table_conf in table_confs]


def parse_columns(columns: dict) -> Iterable[TableDefinition.Column]:
    for col in columns:
        name = col.get("name", None)
        if not name:
            raise ValueError("No name was provided for a column.")
        sql_type = col.get("sql_type", None)
        if not sql_type:
            logger.info("sql_type not set for column: %s, defaulting to TEXT", name)
            # log that varchar is default
        nullability = col.get("nullable", None)
        if nullability:
            nullability = Nullability.NULLABLE
        elif not nullability:
            nullability = Nullability.NOT_NULLABLE
        elif nullability is None:
            logger.info("Nullability defaulting to NOT_NULLABLE for column: %s", name)
            nullability = Nullability.NOT_NULLABLE
            # log that default is not nullable
        # don't check for collation presence because None is typical default for binary
        collation = col.get("collation", None)
        sql_type = SqlType(TypeTag[sql_type])
        yield TableDefinition.Column(name, sql_type, nullability, collation)


def parse_table_definition(tableconf: dict) -> TableDefinition:
    table_name = tableconf.get("table_name", None)
    columns = tableconf.get("columns", [])
    columns = parse_columns(columns)
    table_def = TableDefinition(TableName(table_name), columns)
    constraints = tableconf.get("constraints")
    if constraints is not None:
        if isinstance(constraints, str):
            constraints = [constraints]
        table_def.th_alterations = [
            f'ALTER TABLE "{table_name}" ADD {x}' for x in constraints
        ]
    return table_def


def copy_csv_to_hyper(
    save_path: Path,
    csv: Union[Path, list[Path]],
    schema: Union[TableDefinition, list[TableDefinition]],
    delimiter: str = ",",
    null: str = "",
    encoding: str = "utf-8",
    on_cast_failure: str = "error",
    header: bool = False,
    quote: str = '"',
    escape: Optional[str] = None,
    force_not_null: Optional[list[str]] = None,
    force_null: Optional[list[str]] = None,
) -> Optional[int]:
    """
    :param save_path: where to save the resulting hyperfile
    :param csv: the csv or list of csv paths to be imported
    :param schema: the TableDefinition for the destination hyperfile
    :param delimiter: the separator character for the input csv
    :param null: the string which represents a null in the csv
    :param encoding: the encoding type of the csv
        { 'utf-8' | 'utf-16' | 'utf-16-le' | 'utf-16-be' }
    :param on_cast_failure: how to handle a failure to cast types
        { 'error' | 'set_null' }
    :param header: if a header is present in the csv
    :param quote: character used to mark quoted fields
    :param escape: Specifies the character that should appear
        before a data character that matches the QUOTE value.
        The default is the same as the QUOTE value
        (so that the quoting character is doubled if it appears
        in the data). This must be a single one-byte character.
    :param force_not_null: Do not match the specified columns'
        values against the null string. In the default case
        where the null string is empty, this means that empty
        values will be read as zero-length strings rather than
        nulls, even when they are not quoted.
    :param force_null: Match the specified columns' values
        against the null string, even if it has been quoted,
        and if a match is found set the value to NULL.
        In the default case where the null string is empty,
        this converts a quoted empty string into NULL.
    :return: the count of rows written to the hyper file,
        otherwise None
    """  # noqa
    sql_commands: list[str] = []
    alterations: list[str] = []

    if isinstance(csv, str):
        raise TypeError("str instead of Path object passed as csv")
    if isinstance(csv, Path):
        csv = [csv]
    if isinstance(schema, TableDefinition):
        schema = [schema]
    for c, s in zip(csv, schema):
        if not c.exists():
            raise FileNotFoundError(str(c))
        sql_from = f"COPY {s.table_name} "
        sql_from += f"FROM {escape_string_literal(str(c))}"

        sql_with = _gen_sql_for_with_csv(
            delimiter=delimiter,
            null=null,
            encoding=encoding,
            on_cast_failure=on_cast_failure,
            header=header,
            quote=quote,
            escape=escape,
            force_not_null=force_not_null,
            force_null=force_null,
        )
        sql_commands.append(f"{sql_from} {sql_with}")
        try:
            if s.th_alterations is not None:
                [alterations.append(x) for x in s.th_alterations]
        except AttributeError:
            # alterations are optional
            pass

    process_parameters = {}
    log_file_max_count = os.getenv("TABLEAU_HYPERAPI_LOG_FILE_MAX_COUNT", "3")
    process_parameters["log_file_max_count"] = log_file_max_count
    log_file_size_limit = os.getenv("TABLEAU_HYPERAPI_LOG_FILE_SIZE_LIMIT", "100M")
    process_parameters["log_file_size_limit"] = log_file_size_limit
    log_dir = os.getenv("TABLEAU_HYPERAPI_LOG_DIR", None)
    if log_dir is not None:
        process_parameters["log_dir"] = log_dir

    with HyperProcess(
        telemetry=Telemetry.DO_NOT_SEND_USAGE_DATA_TO_TABLEAU,
        parameters=process_parameters,
    ) as hyper_process:
        with Connection(
            endpoint=hyper_process.endpoint,
            database=save_path,
            create_mode=CreateMode.CREATE_AND_REPLACE,
        ) as connection:
            rowcount = 0
            for s, command in zip(schema, sql_commands):
                connection.catalog.create_table(table_definition=s)
                rowcount += connection.execute_command(command)
            for command in alterations:
                connection.execute_command(command)
            return rowcount



from tableau_helpers.cli import logs
from tableau_helpers import hyper
from tableau_helpers import server
from tableau_helpers import utils as th_utils
import logging
import os
from pathlib import Path
from tempfile import TemporaryDirectory
from importlib import resources


class EnvVarNotSet(Exception):
    """
    Exception to be thrown when a required envvar is not set.
    """


def literal(value: str) -> str:
    return value.encode().decode("unicode_escape")


def main():
    loglevel = os.getenv("LOGLEVEL", logging.WARNING)
    loglevel_stdout = os.getenv("LOGLEVEL_STDOUT", None)
    loglevel_stderr = os.getenv("LOGLEVEL_STDERR", logging.WARNING)
    logs.config_logs(loglevel, loglevel_stdout, loglevel_stderr)

    csv_path = os.getenv("TABLEAU_SOURCE_CSV")
    if csv_path is None:
        raise EnvVarNotSet("TABLEAU_SOURCE_CSV")
    csv_path = th_utils.path_or_url(csv_path)
    tabledef_path = os.getenv("TABLEAU_SOURCE_TABLEDEF")

    if tabledef_path is None:
        raise EnvVarNotSet("TABLEAU_SOURCE_TABLEDEF")
    tabledef_path = Path(tabledef_path)
    tableau_project_path = os.getenv("TABLEAU_DEST_PROJECT")
    tabledef = hyper.load_table_def(tabledef_path)
    tabledef_name = tabledef_path.name.split(".", 1)[0]

    if tableau_project_path is None:
        raise EnvVarNotSet("TABLEAU_DEST_PROJECT")

    csv_delimiter = os.getenv("TABLEAU_SOURCE_CSV_DELIMITER", ",")
    csv_delimiter = literal(csv_delimiter)

    with TemporaryDirectory() as tempdir:
        hyperfile_path = Path(tempdir, tabledef_name + ".hyper")
        hyper.copy_csv_to_hyper(
            save_path=hyperfile_path,
            csv=csv_path,
            schema=tabledef,
            delimiter="\t",
            header=True,
            null="NA",
        )
        server.create_or_replace_hyper_file(
            hyperfile_path=hyperfile_path, project=tableau_project_path
        )


if __name__ == "__main__":
    main()
